@@ -19,12 +19,11 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: FbCommands.cc 4992 2007-07-24 12:12:45Z markt $
-
 #include "FbCommands.hh"
 #include "fluxbox.hh"
 #include "Screen.hh"
 #include "CommandDialog.hh"
+#include "FocusControl.hh"
 #include "Workspace.hh"
 #include "Window.hh"
 #include "Keys.hh"
@@ -32,6 +31,9 @@
 
 #include "FbTk/Theme.hh"
 #include "FbTk/Menu.hh"
+#include "FbTk/CommandParser.hh"
+#include "FbTk/StringUtil.hh"
+#include "FbTk/stringstream.hh"
 
 #include <sys/types.h>
 #include <unistd.h>
@@ -71,18 +73,15 @@ namespace {
 
 void showMenu(const BScreen &screen, FbTk::Menu &menu) {
 
-    // special case for root menu
-    if (&menu == &screen.rootMenu()) {
-        Fluxbox* fb = Fluxbox::instance();
-        if(fb->menuTimestampsChanged()) {
-            // we dont show the menu here because fluxbox
-            // will bring up the rootmenu after the timed
-            // reread of the menu
-            fb->rereadMenu(true);
-            return;
-        }
+    // check if menu has changed
+    if (typeid(menu) == typeid(FbMenu)) {
+        FbMenu *fbmenu = static_cast<FbMenu *>(&menu);
+        if (fbmenu->reloadHelper())
+            fbmenu->reloadHelper()->checkReload();
     }
 
+    FbMenu::setWindow(FocusControl::focusedFbWindow());
+
     Window root_ret; // not used
     Window window_ret; // not used
 
@@ -119,6 +118,12 @@ void showMenu(const BScreen &screen, FbTk::Menu &menu) {
 
 namespace FbCommands {
 
+using FbTk::Command;
+
+REGISTER_UNTRUSTED_COMMAND_WITH_ARGS(exec, FbCommands::ExecuteCmd, void);
+REGISTER_UNTRUSTED_COMMAND_WITH_ARGS(execute, FbCommands::ExecuteCmd, void);
+REGISTER_UNTRUSTED_COMMAND_WITH_ARGS(execcommand, FbCommands::ExecuteCmd, void);
+
 ExecuteCmd::ExecuteCmd(const string &cmd, int screen_num):m_cmd(cmd), m_screen_num(screen_num) {
 
 }
@@ -163,20 +168,32 @@ int ExecuteCmd::run() {
     setsid();
     putenv(const_cast<char *>(displaystring.c_str()));
     execl(shell, shell, "-c", m_cmd.c_str(), static_cast<void*>(NULL));
-    exit(0);
+    exit(EXIT_SUCCESS);
 
     return pid; // compiler happy -> we are happy ;)
 }
 
-SetModKeyCmd::SetModKeyCmd(const string& modkey) : m_modkey(modkey) { }
+FbTk::Command<void> *ExportCmd::parse(const string &command, const string &args,
+                                bool trusted) {
+    string name = args;
+    FbTk::StringUtil::removeFirstWhitespace(name);
+    if (command != "setresourcevalue")
+        FbTk::StringUtil::removeTrailingWhitespace(name);
+    size_t pos = name.find_first_of(command == "export" ? "=" : " \t");
+    if (pos == string::npos || pos == name.size() || !trusted)
+        return 0;
 
-void SetModKeyCmd::execute() {
-    Fluxbox::instance()->setModKey(m_modkey.c_str());
-    Fluxbox::instance()->save_rc();
-    // TODO: we need a better way to do this ...
-    Fluxbox::instance()->reconfigure();
+    string value = name.substr(pos + 1);
+    name = name.substr(0, pos);
+    if (command == "setresourcevalue")
+        return new SetResourceValueCmd(name, value);
+    return new ExportCmd(name, value);
 }
 
+REGISTER_COMMAND_PARSER(setenv, ExportCmd::parse, void);
+REGISTER_COMMAND_PARSER(export, ExportCmd::parse, void);
+REGISTER_COMMAND_PARSER(setresourcevalue, ExportCmd::parse, void);
+
 ExportCmd::ExportCmd(const string& name, const string& value) :
     m_name(name), m_value(value) {
 }
@@ -213,15 +230,28 @@ void ExportCmd::execute() {
     }
 }
 
+REGISTER_COMMAND(exit, FbCommands::ExitFluxboxCmd, void);
+REGISTER_COMMAND(quit, FbCommands::ExitFluxboxCmd, void);
 
 void ExitFluxboxCmd::execute() {
     Fluxbox::instance()->shutdown();
 }
 
+REGISTER_COMMAND(saverc, FbCommands::SaveResources, void);
+
 void SaveResources::execute() {
     Fluxbox::instance()->save_rc();
 }
 
+REGISTER_COMMAND_PARSER(restart, RestartFluxboxCmd::parse, void);
+
+FbTk::Command<void> *RestartFluxboxCmd::parse(const string &command,
+        const string &args, bool trusted) {
+    if (!trusted && !args.empty())
+        return 0;
+    return new RestartFluxboxCmd(args);
+}
+
 RestartFluxboxCmd::RestartFluxboxCmd(const string &cmd):m_cmd(cmd){
 }
 
@@ -232,27 +262,36 @@ void RestartFluxboxCmd::execute() {
         Fluxbox::instance()->restart(m_cmd.c_str());
 }
 
+REGISTER_COMMAND(reconfigure, FbCommands::ReconfigureFluxboxCmd, void);
+REGISTER_COMMAND(reconfig, FbCommands::ReconfigureFluxboxCmd, void);
+
 void ReconfigureFluxboxCmd::execute() {
     Fluxbox::instance()->reconfigure();
 }
 
+REGISTER_COMMAND(reloadstyle, FbCommands::ReloadStyleCmd, void);
 
 void ReloadStyleCmd::execute() {
     SetStyleCmd cmd(Fluxbox::instance()->getStyleFilename());
     cmd.execute();
 }
 
+REGISTER_COMMAND_WITH_ARGS(setstyle, FbCommands::SetStyleCmd, void);
+
 SetStyleCmd::SetStyleCmd(const string &filename):m_filename(filename) {
 
 }
 
 void SetStyleCmd::execute() {
-    Fluxbox::instance()->saveStyleFilename(m_filename.c_str());
-    Fluxbox::instance()->save_rc();
-    FbTk::ThemeManager::instance().load(m_filename,
-                                        Fluxbox::instance()->getStyleOverlayFilename());
+    if (FbTk::ThemeManager::instance().load(m_filename,
+        Fluxbox::instance()->getStyleOverlayFilename())) {
+        Fluxbox::instance()->saveStyleFilename(m_filename.c_str());
+        Fluxbox::instance()->save_rc();
+    }
 }
 
+REGISTER_COMMAND_WITH_ARGS(keymode, FbCommands::KeyModeCmd, void);
+
 KeyModeCmd::KeyModeCmd(const string &arguments):m_keymode(arguments),m_end_args("None Escape") {
     string::size_type second_pos = m_keymode.find_first_of(" \t", 0);
     if (second_pos != string::npos) {
@@ -268,30 +307,71 @@ void KeyModeCmd::execute() {
     Fluxbox::instance()->keys()->keyMode(m_keymode);
 }
 
+REGISTER_COMMAND(hidemenus, FbCommands::HideMenuCmd, void);
+
 void HideMenuCmd::execute() {
-    BScreen *screen = Fluxbox::instance()->mouseScreen();
-    screen->hideMenus();
-    if (screen->rootMenu().isVisible())
-        screen->rootMenu().hide();
-    if (screen->workspaceMenu().isVisible())
-        screen->workspaceMenu().hide();
-    if (FbTk::Menu::shownMenu())
-        FbTk::Menu::shownMenu()->hide();
+    FbTk::Menu::hideShownMenu();
 }
 
+FbTk::Command<void> *ShowClientMenuCmd::parse(const string &command,
+                                        const string &args, bool trusted) {
+    int opts;
+    string pat;
+    FocusableList::parseArgs(args, opts, pat);
+    return new ShowClientMenuCmd(opts, pat);
+}
+
+REGISTER_COMMAND_PARSER(clientmenu, ShowClientMenuCmd::parse, void);
+
+void ShowClientMenuCmd::execute() {
+    BScreen *screen = Fluxbox::instance()->mouseScreen();
+    if (screen == 0)
+        return;
+
+    // TODO: ClientMenu only accepts lists of FluxboxWindows for now
+    //       when that's fixed, use a FocusableList for m_list
+    const FocusableList *list =
+        FocusableList::getListFromOptions(*screen, m_option);
+    m_list.clear();
+    FocusControl::Focusables::const_iterator it = list->clientList().begin(),
+                                             it_end = list->clientList().end();
+    for (; it != it_end; ++it) {
+        if (typeid(**it) == typeid(FluxboxWindow) && m_pat.match(**it))
+            m_list.push_back(static_cast<FluxboxWindow *>(*it));
+    }
+
+    m_menu.reset(new ClientMenu(*screen, m_list, 0));
+    ::showMenu(*screen, *m_menu.get());
+}
+
+REGISTER_COMMAND_WITH_ARGS(custommenu, FbCommands::ShowCustomMenuCmd, void);
+
 ShowCustomMenuCmd::ShowCustomMenuCmd(const string &arguments) : custom_menu_file(arguments) {}
 
 void ShowCustomMenuCmd::execute() {
     BScreen *screen = Fluxbox::instance()->mouseScreen();
     if (screen == 0)
         return;
-    m_menu = MenuCreator::createFromFile(custom_menu_file,
-            screen->screenNumber(), true);
-    if (!m_menu.get())
-        return;
-    ::showMenu(*screen, **m_menu);
+
+    if (!m_menu.get() || screen->screenNumber() != m_menu->screenNumber()) {
+        m_menu.reset(screen->createMenu(""));
+        m_menu->setReloadHelper(new FbTk::AutoReloadHelper());
+        m_menu->reloadHelper()->setReloadCmd(FbTk::RefCount<FbTk::Command<void> >(new FbTk::SimpleCommand<ShowCustomMenuCmd>(*this, &ShowCustomMenuCmd::reload)));
+        m_menu->reloadHelper()->setMainFile(custom_menu_file);
+    } else
+        m_menu->reloadHelper()->checkReload();
+
+    ::showMenu(*screen, *m_menu.get());
 }
 
+void ShowCustomMenuCmd::reload() {
+    m_menu->removeAll();
+    m_menu->setLabel("");
+    MenuCreator::createFromFile(custom_menu_file, *m_menu.get(), m_menu->reloadHelper());
+}
+
+REGISTER_COMMAND(rootmenu, FbCommands::ShowRootMenuCmd, void);
+
 void ShowRootMenuCmd::execute() {
     BScreen *screen = Fluxbox::instance()->mouseScreen();
     if (screen == 0)
@@ -300,6 +380,8 @@ void ShowRootMenuCmd::execute() {
     ::showMenu(*screen, screen->rootMenu());
 }
 
+REGISTER_COMMAND(workspacemenu, FbCommands::ShowWorkspaceMenuCmd, void);
+
 void ShowWorkspaceMenuCmd::execute() {
     BScreen *screen = Fluxbox::instance()->mouseScreen();
     if (screen == 0)
@@ -308,10 +390,13 @@ void ShowWorkspaceMenuCmd::execute() {
     ::showMenu(*screen, screen->workspaceMenu());
 }
 
-
+REGISTER_COMMAND_WITH_ARGS(setworkspacename, FbCommands::SetWorkspaceNameCmd, void);
 
 SetWorkspaceNameCmd::SetWorkspaceNameCmd(const string &name, int spaceid):
-    m_name(name), m_workspace(spaceid) { }
+    m_name(name), m_workspace(spaceid) {
+    if (name.empty())
+        m_name = "empty";
+}
 
 void SetWorkspaceNameCmd::execute() {
     BScreen *screen = Fluxbox::instance()->mouseScreen();
@@ -334,6 +419,8 @@ void SetWorkspaceNameCmd::execute() {
     Fluxbox::instance()->save_rc();
 }
 
+REGISTER_COMMAND(setworkspacenamedialog, FbCommands::WorkspaceNameDialogCmd, void);
+
 void WorkspaceNameDialogCmd::execute() {
 
     BScreen *screen = Fluxbox::instance()->mouseScreen();
@@ -345,6 +432,8 @@ void WorkspaceNameDialogCmd::execute() {
     win->show();
 }
 
+REGISTER_COMMAND(commanddialog, FbCommands::CommandDialogCmd, void);
+
 void CommandDialogCmd::execute() {
     BScreen *screen = Fluxbox::instance()->mouseScreen();
     if (screen == 0)
@@ -370,6 +459,8 @@ void SetResourceValueCmd::execute() {
     Fluxbox::instance()->save_rc();
 }
 
+REGISTER_COMMAND(setresourcevaluedialog, FbCommands::SetResourceValueDialogCmd, void);
+
 void SetResourceValueDialogCmd::execute() {
     BScreen *screen = Fluxbox::instance()->mouseScreen();
     if (screen == 0)
@@ -379,6 +470,8 @@ void SetResourceValueDialogCmd::execute() {
     win->show();
 };
 
+REGISTER_UNTRUSTED_COMMAND_WITH_ARGS(bindkey, FbCommands::BindKeyCmd, void);
+
 BindKeyCmd::BindKeyCmd(const string &keybind):m_keybind(keybind) { }
 
 void BindKeyCmd::execute() {
@@ -392,6 +485,41 @@ void BindKeyCmd::execute() {
     }
 }
 
+FbTk::Command<void> *DeiconifyCmd::parse(const string &command, const string &args,
+                                   bool trusted) {
+    FbTk_istringstream iss(args.c_str());
+    string mode;
+    string d;
+    Destination dest;
+
+    iss >> mode;
+    if (iss.fail())
+        mode="lastworkspace";
+    mode= FbTk::StringUtil::toLower(mode);
+
+    iss >> d;
+    if (iss.fail())
+        d="current";
+    d = FbTk::StringUtil::toLower(d);
+    if (d == "origin" )
+        dest = ORIGIN;
+    else if (d == "originquiet")
+        dest = ORIGINQUIET;
+    else
+        dest = CURRENT;
+
+    if (mode == "all")
+        return new DeiconifyCmd(DeiconifyCmd::ALL, dest);
+    else if (mode == "allworkspace")
+        return new DeiconifyCmd(DeiconifyCmd::ALLWORKSPACE, dest);
+    else if (mode == "last")
+        return new DeiconifyCmd(DeiconifyCmd::LAST, dest);
+    // lastworkspace, default
+    return new DeiconifyCmd(DeiconifyCmd::LASTWORKSPACE, dest);
+}
+
+REGISTER_COMMAND_PARSER(deiconify, DeiconifyCmd::parse, void);
+
 DeiconifyCmd::DeiconifyCmd(Mode mode,
                            Destination dest) : m_mode(mode), m_dest(dest) { }
 
@@ -403,8 +531,8 @@ void DeiconifyCmd::execute() {
     // we need to make a copy of the list of icons, or else our iterator can
     // become invalid
     BScreen::Icons icon_list = screen->iconList();
-    BScreen::Icons::iterator it = icon_list.begin();
-    BScreen::Icons::iterator itend= icon_list.end();
+    BScreen::Icons::reverse_iterator it = icon_list.rbegin();
+    BScreen::Icons::reverse_iterator itend= icon_list.rend();
     unsigned int workspace_num= screen->currentWorkspaceID();
     unsigned int old_workspace_num;
 
@@ -420,8 +548,7 @@ void DeiconifyCmd::execute() {
                 (*it)->isStuck()) {
                 if (m_dest == ORIGIN || m_dest == ORIGINQUIET)
                     screen->sendToWorkspace(old_workspace_num, (*it), change_ws);
-                else
-                    (*it)->deiconify(false);
+                (*it)->deiconify();
             }
         }
         break;
@@ -437,7 +564,7 @@ void DeiconifyCmd::execute() {
                     m_mode != LASTWORKSPACE)
                     screen->sendToWorkspace(old_workspace_num, (*it), change_ws);
                 else
-                    (*it)->deiconify(false);
+                    (*it)->deiconify();
                 break;
             }
         }
@@ -445,4 +572,4 @@ void DeiconifyCmd::execute() {
     };
 }
 
-}; // end namespace FbCommands
+} // end namespace FbCommands
