@@ -22,11 +22,10 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: ImageControl.cc 4857 2007-04-17 07:52:40Z mathias $
-
 #include "ImageControl.hh"
 
 #include "TextureRender.hh"
+#include "Texture.hh"
 #include "App.hh"
 #include "SimpleCommand.hh"
 #include "I18n.hh"
@@ -69,15 +68,17 @@ using std::list;
 
 namespace FbTk {
 
-// lookup table for texture
-unsigned long *ImageControl::sqrt_table = 0;
+namespace { // anonymous
+
+static unsigned long *sqrt_table = 0; /// lookup table
+
 #ifdef TIMEDCACHE
-bool ImageControl::s_timed_cache = true;
+bool s_timed_cache = true;
 #else
-bool ImageControl::s_timed_cache = false;
+bool s_timed_cache = false;
 #endif // TIMEDCACHE
 
-namespace { // anonymous
+
 
 inline unsigned long bsqrt(unsigned long x) {
     if (x <= 0) return 0;
@@ -95,9 +96,16 @@ inline unsigned long bsqrt(unsigned long x) {
 
 }; // end anonymous namespace
 
-ImageControl::ImageControl(int screen_num, bool dither,
+struct ImageControl::Cache {
+    Pixmap pixmap;
+    Pixmap texture_pixmap;
+    Orientation orient;
+    unsigned int count, width, height;
+    unsigned long pixel1, pixel2, texture;
+};
+
+ImageControl::ImageControl(int screen_num,
                            int cpc, unsigned long cache_timeout, unsigned long cmax):
-    m_dither(dither),
     m_colors(0),
     m_num_colors(0),
     m_colors_per_channel(cpc) {
@@ -114,7 +122,7 @@ ImageControl::ImageControl(int screen_num, bool dither,
 
     if (cache_timeout && s_timed_cache) {
         m_timer.setTimeout(cache_timeout);
-        RefCount<Command> clean_cache(new SimpleCommand<ImageControl>(*this, &ImageControl::cleanCache));
+        RefCount<Command<void> > clean_cache(new SimpleCommand<ImageControl>(*this, &ImageControl::cleanCache));
         m_timer.setCommand(clean_cache);
         m_timer.start();
     }
@@ -150,7 +158,7 @@ ImageControl::~ImageControl() {
         delete [] m_colors;
     }
 
-    if (cache.size() > 0) {
+    if (!cache.empty()) {
         CacheList::iterator it = cache.begin();
         CacheList::iterator it_end = cache.end();
         for (; it != it_end; ++it) {
@@ -309,13 +317,6 @@ void ImageControl::colorTables(const unsigned char **rmt, const unsigned char **
     if (bbit) *bbit = blue_bits;
 }
 
-#ifdef NOT_USED
-void ImageControl::getXColorTable(XColor **c, int *n) {
-    if (c) *c = m_colors;
-    if (n) *n = m_num_colors;
-}
-#endif
-
 void ImageControl::getGradientBuffers(unsigned int w,
                                       unsigned int h,
                                       unsigned int **xbuf,
@@ -371,15 +372,6 @@ void ImageControl::installRootColormap() {
     XUngrabServer(disp);
 }
 
-#ifdef NOT_USED
-void ImageControl::setColorsPerChannel(int cpc) {
-    if (cpc < 2) cpc = 2;
-    if (cpc > 6) cpc = 6;
-
-    m_colors_per_channel = cpc;
-}
-#endif
-
 unsigned long ImageControl::getSqrt(unsigned int x) const {
     if (! sqrt_table) {
         // build sqrt table for use with elliptic gradient
@@ -440,8 +432,6 @@ void ImageControl::createColorTable() {
 
     if (bits_per_pixel == 0)
         bits_per_pixel = m_screen_depth;
-    if (bits_per_pixel >= 24)
-        setDither(false);
 
     red_offset = green_offset = blue_offset = 0;
 
@@ -492,12 +482,7 @@ void ImageControl::createColorTable() {
 
         m_colors = new XColor[m_num_colors];
 
-        int bits = 256 / m_colors_per_channel;
-
-#ifndef ORDEREDPSEUDO
-        bits = 255 / (m_colors_per_channel - 1);
-#endif // ORDEREDPSEUDO
-
+        int bits = 255 / (m_colors_per_channel - 1);
         red_bits = green_bits = blue_bits = bits;
 
         for (unsigned int i = 0; i < 256; i++) {
@@ -510,7 +495,7 @@ void ImageControl::createColorTable() {
                 for (int b = 0; b < m_colors_per_channel; b++, i++) {
                     m_colors[i].red = (r * 0xffff) / (m_colors_per_channel - 1);
                     m_colors[i].green = (g * 0xffff) / (m_colors_per_channel - 1);
-                    m_colors[i].blue = (b * 0xffff) / (m_colors_per_channel - 1);;
+                    m_colors[i].blue = (b * 0xffff) / (m_colors_per_channel - 1);
                     m_colors[i].flags = DoRed|DoGreen|DoBlue;
                 }
             }
@@ -602,7 +587,7 @@ void ImageControl::createColorTable() {
             for (unsigned int i = 0; i < m_num_colors; i++) {
                 m_colors[i].red = (i * 0xffff) / (m_colors_per_channel - 1);
                 m_colors[i].green = (i * 0xffff) / (m_colors_per_channel - 1);
-                m_colors[i].blue = (i * 0xffff) / (m_colors_per_channel - 1);;
+                m_colors[i].blue = (i * 0xffff) / (m_colors_per_channel - 1);
                 m_colors[i].flags = DoRed|DoGreen|DoBlue;
 
                 if (! XAllocColor(disp, m_colormap,
@@ -663,4 +648,4 @@ void ImageControl::createColorTable() {
     }
 }
 
-}; // end namespace FbTk
+} // end namespace FbTk
