@@ -1,5 +1,5 @@
 // Xutil.cc for Fluxbox - an X11 Window manager
-// Copyright (c) 2003-2004 Henrik Kinnunen (fluxgen(at)users.sourceforge.net)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //                and Simon Bowden    (rathnor at users.sourceforge.net)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -20,8 +20,6 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Xutil.cc 3700 2004-11-19 11:37:27Z mathias $
-
 #include "Xutil.hh"
 
 #include "FbTk/I18n.hh"
@@ -29,13 +27,25 @@
 
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
-#include <X11/Xlib.h>
 #include <iostream>
-using namespace std;
+
+#ifdef HAVE_CSTRING
+  #include <cstring>
+#else
+  #include <string.h>
+#endif
+
+using std::string;
+using std::strlen;
+
+#ifdef DEBUG
+using std::cerr;
+using std::endl;
+#endif // DEBUG
 
 namespace Xutil {
 
-std::string getWMName(Window window) {
+FbTk::FbString getWMName(Window window) {
 
     if (window == None)
         return "";
@@ -44,36 +54,37 @@ std::string getWMName(Window window) {
 
     XTextProperty text_prop;
     text_prop.value = 0;
-    char **list;
-    int num;
+    char **list = 0;
+    int num = 0;
     _FB_USES_NLS;
-    std::string name;
+    string name;
 
     if (XGetWMName(display, window, &text_prop)) {
         if (text_prop.value && text_prop.nitems > 0) {
             if (text_prop.encoding != XA_STRING) {
-				
+
                 text_prop.nitems = strlen((char *) text_prop.value);
-				
-                if ((XmbTextPropertyToTextList(display, &text_prop,
-                                               &list, &num) == Success) &&
-                    (num > 0) && *list) {
-                    name = static_cast<char *>(*list);
+                XmbTextPropertyToTextList(display, &text_prop, &list, &num);
+
+                if (num > 0 && list != 0)
+                    name = FbTk::FbStringUtil::LocaleStrToFb(static_cast<char *>(*list));
+                else
+                    name = text_prop.value ? FbTk::FbStringUtil::XStrToFb((char *)text_prop.value) : "";
+
+                if (list)
                     XFreeStringList(list);
-                } else
-                    name = text_prop.value ? (char *)text_prop.value : "";
-					
-            } else				
-                name = text_prop.value ? (char *)text_prop.value : "";
+
+            } else
+                name = text_prop.value ? FbTk::FbStringUtil::XStrToFb((char *)text_prop.value) : "";
 
             XFree(text_prop.value);
 
         } else { // default name
-            name = _FBTEXT(Window, Unnamed, "Unnamed", "Default name for a window without a WM_NAME");
+            name = _FB_XTEXT(Window, Unnamed, "Unnamed", "Default name for a window without a WM_NAME");
         }
     } else {
         // default name
-        name = _FBTEXT(Window, Unnamed, "Unnamed", "Default name for a window without a WM_NAME");
+        name = _FB_XTEXT(Window, Unnamed, "Unnamed", "Default name for a window without a WM_NAME");
     }
 
     return name;
@@ -81,19 +92,19 @@ std::string getWMName(Window window) {
 
 
 // The name of this particular instance
-std::string getWMClassName(Window win) {
+string getWMClassName(Window win) {
     XClassHint ch;
-    std::string instance_name;
+    string instance_name;
 
     if (XGetClassHint(FbTk::App::instance()->display(), win, &ch) == 0) {
 #ifdef DEBUG
         cerr<<"Xutil: Failed to read class hint!"<<endl;
 #endif //DEBUG
         instance_name = "";
-    } else {        
+    } else {
 
         XFree(ch.res_class);
-        
+
         if (ch.res_class != 0) {
             instance_name = const_cast<char *>(ch.res_name);
             XFree(ch.res_name);
@@ -107,19 +118,19 @@ std::string getWMClassName(Window win) {
 }
 
 // the name of the general class of the app
-std::string getWMClassClass(Window win) {
+string getWMClassClass(Window win) {
     XClassHint ch;
-    std::string class_name;
+    string class_name;
 
     if (XGetClassHint(FbTk::App::instance()->display(), win, &ch) == 0) {
 #ifdef DEBUG
         cerr<<"Xutil: Failed to read class hint!"<<endl;
 #endif //DEBUG
         class_name = "";
-    } else {        
+    } else {
 
         XFree(ch.res_name);
-        
+
         if (ch.res_class != 0) {
             class_name = const_cast<char *>(ch.res_class);
             XFree(ch.res_class);
@@ -131,5 +142,5 @@ std::string getWMClassClass(Window win) {
     return class_name;
 }
 
-}; // end namespace Xutil
+} // end namespace Xutil
 
