@@ -22,8 +22,6 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: fluxbox.cc 4998 2007-08-04 17:14:13Z simonb $
-
 #include "fluxbox.hh"
 
 #include "Screen.hh"
@@ -65,9 +63,6 @@
 #include "config.h"
 #endif // HAVE_CONFIG_H
 
-#ifdef SLIT
-#include "Slit.hh"
-#endif // SLIT
 #ifdef USE_GNOME
 #include "Gnome.hh"
 #endif // USE_GNOME
@@ -77,11 +72,6 @@
 #ifdef REMEMBER
 #include "Remember.hh"
 #endif // REMEMBER
-#ifdef USE_TOOLBAR
-#include "Toolbar.hh"
-#else
-class Toolbar { };
-#endif // USE_TOOLBAR
 
 // X headers
 #include <X11/Xlib.h>
@@ -189,7 +179,8 @@ int handleXErrors(Display *d, XErrorEvent *e) {
 //static singleton var
 Fluxbox *Fluxbox::s_singleton=0;
 
-Fluxbox::Fluxbox(int argc, char **argv, const char *dpy_name, const char *rcfilename)
+Fluxbox::Fluxbox(int argc, char **argv, const char *dpy_name,
+                 const char *rcfilename, bool xsync)
     : FbTk::App(dpy_name),
       m_fbatoms(new FbAtoms()),
       m_resourcemanager(rcfilename, true),
@@ -206,13 +197,11 @@ Fluxbox::Fluxbox(int argc, char **argv, const char *dpy_name, const char *rcfile
       m_rc_menufile(m_resourcemanager, DEFAULTMENU, "session.menuFile", "Session.MenuFile"),
       m_rc_keyfile(m_resourcemanager, DEFAULTKEYSFILE, "session.keyFile", "Session.KeyFile"),
       m_rc_slitlistfile(m_resourcemanager, "~/." + realProgramName("fluxbox") + "/slitlist", "session.slitlistFile", "Session.SlitlistFile"),
-      m_rc_groupfile(m_resourcemanager, "~/." + realProgramName("fluxbox") + "/groups", "session.groupFile", "Session.GroupFile"),
       m_rc_appsfile(m_resourcemanager, "~/." + realProgramName("fluxbox") + "/apps", "session.appsFile", "Session.AppsFile"),
       m_rc_tabs_attach_area(m_resourcemanager, ATTACH_AREA_WINDOW, "session.tabsAttachArea", "Session.TabsAttachArea"),
       m_rc_cache_life(m_resourcemanager, 5, "session.cacheLife", "Session.CacheLife"),
       m_rc_cache_max(m_resourcemanager, 200, "session.cacheMax", "Session.CacheMax"),
       m_rc_auto_raise_delay(m_resourcemanager, 250, "session.autoRaiseDelay", "Session.AutoRaiseDelay"),
-      m_rc_mod_key(m_resourcemanager, "Mod1", "session.modKey", "Session.ModKey"),
       m_masked_window(0),
       m_mousescreen(0),
       m_keyscreen(0),
@@ -220,7 +209,6 @@ Fluxbox::Fluxbox(int argc, char **argv, const char *dpy_name, const char *rcfile
       m_masked(0),
       m_rc_file(rcfilename ? rcfilename : ""),
       m_argv(argv), m_argc(argc),
-      m_revert_screen(0),
       m_showing_dialog(false),
       m_starting(true),
       m_restarting(false),
@@ -268,7 +256,7 @@ Fluxbox::Fluxbox(int argc, char **argv, const char *dpy_name, const char *rcfile
     // Because when the command is executed we shouldn't do reconfig directly
     // because it could affect ongoing menu stuff so we need to reconfig in
     // the next event "round".
-    FbTk::RefCount<FbTk::Command> reconfig_cmd(new FbTk::SimpleCommand<Fluxbox>(*this, &Fluxbox::timed_reconfigure));
+    FbTk::RefCount<FbTk::Command<void> > reconfig_cmd(new FbTk::SimpleCommand<Fluxbox>(*this, &Fluxbox::timed_reconfigure));
     timeval to;
     to.tv_sec = 0;
     to.tv_usec = 1;
@@ -276,13 +264,8 @@ Fluxbox::Fluxbox(int argc, char **argv, const char *dpy_name, const char *rcfile
     m_reconfig_timer.setCommand(reconfig_cmd);
     m_reconfig_timer.fireOnce(true);
 
-    // set a timer to revert focus on FocusOut, in case no FocusIn arrives
-    FbTk::RefCount<FbTk::Command> revert_cmd(new FbTk::SimpleCommand<Fluxbox>(*this, &Fluxbox::revert_focus));
-    m_revert_timer.setCommand(revert_cmd);
-    m_revert_timer.setTimeout(to);
-    m_revert_timer.fireOnce(true);
-
-    // XSynchronize(disp, True);
+    if (xsync)
+        XSynchronize(disp, True);
 
     s_singleton = this;
     m_have_shape = false;
@@ -320,6 +303,14 @@ Fluxbox::Fluxbox(int argc, char **argv, const char *dpy_name, const char *rcfile
 #endif // HAVE_GETPID
 
 
+    // setup theme manager to have our style file ready to be scanned
+    FbTk::ThemeManager::instance().load(getStyleFilename(), getStyleOverlayFilename());
+
+    // Create keybindings handler and load keys file
+    // Note: this needs to be done before creating screens
+    m_key.reset(new Keys);
+    m_key->reconfigure();
+
     vector<int> screens;
     int i;
 
@@ -335,7 +326,7 @@ Fluxbox::Fluxbox(int argc, char **argv, const char *dpy_name, const char *rcfile
                 cerr << _FB_CONSOLETEXT(main, ScreenRequiresArg, 
                                         "error, -screen requires argument", 
                                         "the -screen option requires a file argument") << endl;
-                exit(1);
+                exit(EXIT_FAILURE);
             }
 
             // "all" is default
@@ -385,19 +376,19 @@ Fluxbox::Fluxbox(int argc, char **argv, const char *dpy_name, const char *rcfile
 
     m_keyscreen = m_mousescreen = m_screen_list.front();
 
-    // parse apps file after creating screens but before creating windows
-#ifdef REMEMBER
-        addAtomHandler(new Remember(), "remember"); // for remembering window attribs
-#endif // REMEMBER
-    // ewmh handler needs to be added after apps file handler, or else some
-    // window properties are set incorrectly on new windows
-    // this dependency should probably be made more robust
 #ifdef USE_NEWWMSPEC
     addAtomHandler(new Ewmh(), "ewmh"); // for Extended window manager atom support
 #endif // USE_NEWWMSPEC
 #ifdef USE_GNOME
     addAtomHandler(new Gnome(), "gnome"); // for gnome 1 atom support
 #endif //USE_GNOME
+    // parse apps file after creating screens (so we can tell if it's a restart
+    // for [startup] items) but before creating windows
+    // this needs to be after ewmh and gnome, so state atoms don't get
+    // overwritten before they're applied
+#ifdef REMEMBER
+    addAtomHandler(new Remember(), "remember"); // for remembering window attribs
+#endif // REMEMBER
 
     // init all "screens"
     ScreenList::iterator it = m_screen_list.begin();
@@ -407,17 +398,10 @@ Fluxbox::Fluxbox(int argc, char **argv, const char *dpy_name, const char *rcfile
 
     XAllowEvents(disp, ReplayPointer, CurrentTime);
 
-    // setup theme manager to have our style file ready to be scanned
-    FbTk::ThemeManager::instance().load(getStyleFilename(), getStyleOverlayFilename());
-
     //XSynchronize(disp, False);
     sync(false);
 
-    m_reconfigure_wait = m_reread_menu_wait = false;
-
-    // Create keybindings handler and load keys file
-    m_key.reset(new Keys);
-    m_key->load(StringUtil::expandFilename(*m_rc_keyfile).c_str());
+    m_reconfigure_wait = false;
 
     m_resourcemanager.unlock();
     ungrab();
@@ -433,33 +417,11 @@ Fluxbox::Fluxbox(int argc, char **argv, const char *dpy_name, const char *rcfile
     //
     //    m_resourcemanager.dump();
 
-#ifdef USE_TOOLBAR
-    // finally, show toolbar
-    Toolbars::iterator toolbar_it = m_toolbars.begin();
-    Toolbars::iterator toolbar_it_end = m_toolbars.end();
-    for (; toolbar_it != toolbar_it_end; ++toolbar_it)
-        (*toolbar_it)->updateVisibleState();
-#endif // USE_TOOLBAR
-
 }
 
 
 Fluxbox::~Fluxbox() {
 
-    // destroy toolbars
-    while (!m_toolbars.empty()) {
-        delete m_toolbars.back();
-        m_toolbars.pop_back();
-    }
-
-    // destroy atomhandlers
-    for (AtomHandlerContainerIt it= m_atomhandler.begin();
-         it != m_atomhandler.end();
-         it++) {
-        delete (*it).first;
-    }
-    m_atomhandler.clear();
-
     // this needs to be destroyed before screens; otherwise, menus stored in
     // key commands cause a segfault when the XLayerItem is destroyed
     m_key.reset(0);
@@ -470,51 +432,26 @@ Fluxbox::~Fluxbox() {
         m_screen_list.pop_back();
     }
 
-
-    clearMenuFilenames();
+    // destroy atomhandlers
+    for (AtomHandlerContainerIt it= m_atomhandler.begin();
+         it != m_atomhandler.end();
+         it++) {
+        delete (*it).first;
+    }
+    m_atomhandler.clear();
 }
 
 
 void Fluxbox::initScreen(BScreen *screen) {
 
-    Display* disp = display();
-
     // now we can create menus (which needs this screen to be in screen_list)
     screen->initMenus();
 
-#ifdef HAVE_GETPID
-    pid_t bpid = getpid();
-
-    screen->rootWindow().changeProperty(getFluxboxPidAtom(), XA_CARDINAL,
-                                        sizeof(pid_t) * 8, PropModeReplace,
-                                        (unsigned char *) &bpid, 1);
-#endif // HAVE_GETPID
-
-#ifdef HAVE_RANDR
-    // setup RANDR for this screens root window
-    // we need to determine if we should use old randr select input function or not
-#ifdef X_RRScreenChangeSelectInput
-    // use old set randr event
-    XRRScreenChangeSelectInput(disp, screen->rootWindow().window(), True);
-#else
-    XRRSelectInput(disp, screen->rootWindow().window(),
-                   RRScreenChangeNotifyMask);
-#endif // X_RRScreenChangeSelectInput
-
-#endif // HAVE_RANDR
-
-
-#ifdef USE_TOOLBAR
-    m_toolbars.push_back(new Toolbar(*screen,
-                                     *screen->layerManager().
-                                     getLayer(::Layer::NORMAL)));
-#endif // USE_TOOLBAR
-
-    // must do this after toolbar is created
     screen->initWindows();
 
     // attach screen signals to this
     screen->currentWorkspaceSig().attach(this);
+    screen->focusedWindowSig().attach(this);
     screen->workspaceCountSig().attach(this);
     screen->workspaceNamesSig().attach(this);
     screen->workspaceAreaSig().attach(this);
@@ -528,10 +465,6 @@ void Fluxbox::initScreen(BScreen *screen) {
     }
 
     FocusControl::revertFocus(*screen); // make sure focus style is correct
-#ifdef SLIT
-    if (screen->slit())
-        screen->slit()->show();
-#endif // SLIT
 
 }
 
@@ -545,8 +478,11 @@ void Fluxbox::eventLoop() {
 
             if (last_bad_window != None && e.xany.window == last_bad_window &&
                 e.type != DestroyNotify) { // we must let the actual destroys through
+                if (e.type == FocusOut)
+                    revertFocus();
 #ifdef DEBUG
-                cerr<<"Fluxbox::eventLoop(): removing bad window from event queue"<<endl;
+                else
+                    cerr<<"Fluxbox::eventLoop(): removing bad window from event queue"<<endl;
 #endif // DEBUG
             } else {
                 last_bad_window = None;
@@ -587,13 +523,16 @@ void Fluxbox::ungrab() {
 */
 void Fluxbox::setupConfigFiles() {
 
-    bool create_init = false, create_keys = false, create_menu = false;
+    bool create_init = false, create_keys = false, create_menu = false,
+         create_apps = false, create_overlay = false, create_windowmenu = false;
 
-    string dirname = getenv("HOME") + string("/.") + m_RC_PATH + "/";
-    string init_file, keys_file, menu_file, slitlist_file;
-    init_file = dirname + m_RC_INIT_FILE;
-    keys_file = dirname + "keys";
-    menu_file = dirname + "menu";
+    string dirname = getDefaultDataFilename("");
+    string init_file = getDefaultDataFilename(m_RC_INIT_FILE);
+    string keys_file = getDefaultDataFilename("keys");
+    string menu_file = getDefaultDataFilename("menu");
+    string apps_file = getDefaultDataFilename("apps");
+    string overlay_file = getDefaultDataFilename("overlay");
+    string windowmenu_file = getDefaultDataFilename("windowmenu");
 
     struct stat buf;
 
@@ -607,6 +546,12 @@ void Fluxbox::setupConfigFiles() {
             create_keys = true;
         if (stat(menu_file.c_str(), &buf))
             create_menu = true;
+        if (stat(apps_file.c_str(), &buf))
+            create_apps = true;
+        if (stat(overlay_file.c_str(), &buf))
+            create_overlay = true;
+        if (stat(windowmenu_file.c_str(), &buf))
+            create_windowmenu = true;
 
     } else {
 #ifdef DEBUG
@@ -624,7 +569,8 @@ void Fluxbox::setupConfigFiles() {
         }
 
         //mark creation of files
-        create_init = create_keys = create_menu = true;
+        create_init = create_keys = create_menu = create_apps = create_overlay =
+            create_windowmenu = true;
     }
 
 
@@ -636,11 +582,22 @@ void Fluxbox::setupConfigFiles() {
     if (create_menu)
         FbTk::FileUtil::copyFile(DEFAULTMENU, menu_file.c_str());
 
+    // copy apps file
+    if (create_apps)
+        FbTk::FileUtil::copyFile(DEFAULT_APPSFILE, apps_file.c_str());
+
+    // copy overlay file
+    if (create_overlay)
+        FbTk::FileUtil::copyFile(DEFAULT_OVERLAY, overlay_file.c_str());
+
     // copy init file
     if (create_init)
         FbTk::FileUtil::copyFile(DEFAULT_INITFILE, init_file.c_str());
 
-#define CONFIG_VERSION 1
+    if (create_windowmenu)
+        FbTk::FileUtil::copyFile(DEFAULT_WINDOWMENU, windowmenu_file.c_str());
+
+#define CONFIG_VERSION 10
     FbTk::Resource<int> config_version(m_resourcemanager, 0,
             "session.configVersion", "Session.ConfigVersion");
     if (*config_version < CONFIG_VERSION) {
@@ -708,30 +665,6 @@ void Fluxbox::handleEvent(XEvent * const e) {
         }
     }
 
-    // we need to check focus out for menus before
-    // we call FbTk eventhandler
-    // so we can get FbTk::Menu::focused() before it sets to 0
-    if (e->type == FocusOut &&
-        e->xfocus.mode != NotifyGrab &&
-        e->xfocus.detail != NotifyPointer &&
-        e->xfocus.detail != NotifyInferior &&
-        FbTk::Menu::focused() != 0 &&
-        FbTk::Menu::focused()->window() == e->xfocus.window) {
-
-        // find screen num
-        ScreenList::iterator it = m_screen_list.begin();
-        ScreenList::iterator it_end = m_screen_list.end();
-        for (; it != it_end; ++it) {
-            if ( (*it)->screenNumber() ==
-                 FbTk::Menu::focused()->fbwindow().screenNumber()) {
-                FocusControl::setFocusedWindow(0);
-                m_revert_screen = *it;
-                m_revert_timer.start();
-                break; // found the screen, no more search
-            }
-        }
-    }
-
     // try FbTk::EventHandler first
     FbTk::EventManager::instance()->handleEvent(*e);
 
@@ -825,7 +758,7 @@ void Fluxbox::handleEvent(XEvent * const e) {
             XRefreshKeyboardMapping(&e->xmapping);
             FbTk::KeyUtil::instance().init(); // reinitialise the key utils
             // reconfigure keys (if the mapping changes, they don't otherwise update
-            m_key->reconfigure(StringUtil::expandFilename(*m_rc_keyfile).c_str());
+            m_key->reload();
         }
         break;
     case CreateNotify:
@@ -836,11 +769,6 @@ void Fluxbox::handleEvent(XEvent * const e) {
             FluxboxWindow *win = winclient->fbwindow();
             if (win)
                 win->destroyNotifyEvent(e->xdestroywindow);
-
-            delete winclient;
-
-            if (win && win->numClients() == 0)
-                delete win;
         }
 
     }
@@ -900,9 +828,18 @@ void Fluxbox::handleEvent(XEvent * const e) {
             e->xfocus.detail == NotifyPointer ||
             e->xfocus.detail == NotifyInferior)
             break;
+
+        if (FbTk::Menu::focused() &&
+            FbTk::Menu::focused()->window() == e->xfocus.window) {
+            m_keyscreen = findScreen(FbTk::Menu::focused()->screenNumber());
+            FocusControl::setFocusedWindow(0);
+            break;
+        }
+
         WinClient *winclient = searchWindow(e->xfocus.window);
-        if (winclient && FocusControl::focusedWindow() != winclient)
-            FocusControl::setFocusedWindow(winclient);
+        if (winclient)
+            m_keyscreen = &winclient->screen();
+        FocusControl::setFocusedWindow(winclient);
 
     } break;
     case FocusOut:{
@@ -913,19 +850,12 @@ void Fluxbox::handleEvent(XEvent * const e) {
             break;
 
         WinClient *winclient = searchWindow(e->xfocus.window);
-        if (winclient == 0 && FbTk::Menu::focused() == 0) {
-#ifdef DEBUG
-            cerr<<__FILE__<<"("<<__FUNCTION__<<") Focus out is not a FluxboxWindow !!"<<endl;
-#endif // DEBUG
-        } else if (winclient && (winclient == FocusControl::focusedWindow() ||
-                                 FocusControl::focusedWindow() == 0) &&
-                   (winclient->fbwindow() == 0
-                    || !winclient->fbwindow()->isMoving())) {
+        if ((winclient == FocusControl::focusedWindow() ||
+             FocusControl::focusedWindow() == 0) &&
             // we don't unfocus a moving window
-            FocusControl::setFocusedWindow(0);
-            m_revert_screen = &winclient->screen();
-            m_revert_timer.start();
-        }
+            (!winclient || !winclient->fbwindow() ||
+             !winclient->fbwindow()->isMoving()))
+            revertFocus();
     }
 	break;
     case ClientMessage:
@@ -935,6 +865,9 @@ void Fluxbox::handleEvent(XEvent * const e) {
 
 #ifdef HAVE_RANDR
         if (e->type == m_randr_event_type) {
+#ifdef HAVE_RANDR1_2
+            XRRUpdateConfiguration(e);
+#endif
             // update root window size in screen
             BScreen *scr = searchScreen(e->xany.window);
             if (scr != 0)
@@ -955,30 +888,19 @@ void Fluxbox::handleUnmapNotify(XUnmapEvent &ue) {
         return;
     }
 
-    WinClient *winclient = 0;
+    WinClient *winclient = searchWindow(ue.window);
 
-    if ((winclient = searchWindow(ue.window)) != 0) {
+    if (winclient != 0) {
 
-        if (winclient != 0) {
-            FluxboxWindow *win = winclient->fbwindow();
-
-            if (!win) {
-                delete winclient;
-                return;
-            }
-
-            // this should delete client and adjust m_focused_window if necessary
-            win->unmapNotifyEvent(ue);
-
-            winclient = 0; // it's invalid now when win destroyed the client
-
-            // finally destroy window if empty
-            if (win->numClients() == 0) {
-                delete win;
-                win = 0;
-            }
+        FluxboxWindow *win = winclient->fbwindow();
+        if (!win) {
+            delete winclient;
+            return;
         }
 
+        // this should delete client and adjust m_focused_window if necessary
+        win->unmapNotifyEvent(ue);
+
     // according to http://tronche.com/gui/x/icccm/sec-4.html#s-4.1.4
     // a XWithdrawWindow is
     //   1) unmapping the window (which leads to the upper branch
@@ -1019,41 +941,6 @@ void Fluxbox::handleClientMessage(XClientMessageEvent &ce) {
             winclient->fbwindow()->iconify();
         if (ce.data.l[0] == NormalState)
             winclient->fbwindow()->deiconify();
-    } else if (ce.message_type == m_fbatoms->getFluxboxChangeWorkspaceAtom()) {
-        BScreen *screen = searchScreen(ce.window);
-
-        if (screen && ce.data.l[0] >= 0 &&
-            ce.data.l[0] < (signed)screen->numberOfWorkspaces())
-            screen->changeWorkspaceID(ce.data.l[0]);
-
-    } else if (ce.message_type == m_fbatoms->getFluxboxChangeWindowFocusAtom()) {
-        WinClient *winclient = searchWindow(ce.window);
-        if (winclient) {
-            FluxboxWindow *win = winclient->fbwindow();
-            if (win && win->isVisible())
-                win->setCurrentClient(*winclient, true);
-        }
-    } else if (ce.message_type == m_fbatoms->getFluxboxCycleWindowFocusAtom()) {
-        BScreen *screen = searchScreen(ce.window);
-
-        if (screen) {
-            if (! ce.data.l[0])
-                screen->focusControl().prevFocus();
-            else
-                screen->focusControl().nextFocus();
-        }
-    } else if (ce.message_type == m_fbatoms->getFluxboxChangeAttributesAtom()) {
-        WinClient *winclient = searchWindow(ce.window);
-        FluxboxWindow *win = 0;
-        if (winclient && (win = winclient->fbwindow()) && winclient->validateClient()) {
-            FluxboxWindow::BlackboxHints net;
-            net.flags = ce.data.l[0];
-            net.attrib = ce.data.l[1];
-            net.workspace = ce.data.l[2];
-            net.stack = ce.data.l[3];
-            net.decoration = static_cast<int>(ce.data.l[4]);
-            win->changeBlackboxHints(net);
-        }
     } else {
         WinClient *winclient = searchWindow(ce.window);
         BScreen *screen = searchScreen(ce.window);
@@ -1098,7 +985,7 @@ void Fluxbox::handleSignal(int signum) {
         break;
     default:
         fprintf(stderr,
-                _FB_CONSOLETEXT(BaseDisplay, SignalCaught, "%s:      signal %d caught\n", "signal catch debug message. Include %s for command and %d for signal number").c_str(),
+                _FB_CONSOLETEXT(BaseDisplay, SignalCaught, "%s:      signal %d caught\n", "signal catch debug message. Include %s for Command<void> and %d for signal number").c_str(),
                 m_argv[0], signum);
 
         if (! m_starting && ! re_enter) {
@@ -1117,73 +1004,93 @@ void Fluxbox::handleSignal(int signum) {
 
 void Fluxbox::update(FbTk::Subject *changedsub) {
     //TODO: fix signaling, this does not look good
+    FluxboxWindow *fbwin = 0;
+    WinClient *client = 0;
+
     if (typeid(*changedsub) == typeid(FluxboxWindow::WinSubject)) {
         FluxboxWindow::WinSubject *winsub = dynamic_cast<FluxboxWindow::WinSubject *>(changedsub);
-        FluxboxWindow &win = winsub->win();
-        if ((&(win.hintSig())) == changedsub) { // hint signal
-            for (AtomHandlerContainerIt it= m_atomhandler.begin();
-                 it != m_atomhandler.end(); ++it) {
-                if ( (*it).first->update())
-                    (*it).first->updateHints(win);
-            }
-        } else if ((&(win.stateSig())) == changedsub) { // state signal
-            for (AtomHandlerContainerIt it= m_atomhandler.begin();
-                 it != m_atomhandler.end(); ++it) {
-                if ((*it).first->update())
-                    (*it).first->updateState(win);
-            }
-            // if window changed to iconic state
-            // add to icon list
-            if (win.isIconic()) {
-                win.screen().addIcon(&win);
-                Workspace *space = win.screen().getWorkspace(win.workspaceNumber());
-                if (space != 0)
-                    space->removeWindow(&win, true);
-            }
+        fbwin = &winsub->win();
+    } else if (typeid(*changedsub) == typeid(Focusable::FocusSubject)) {
+        Focusable::FocusSubject *winsub = dynamic_cast<Focusable::FocusSubject *>(changedsub);
+        fbwin = winsub->win().fbwindow();
+        if (typeid(winsub->win()) == typeid(WinClient))
+            client = dynamic_cast<WinClient *>(&winsub->win());
+    }
 
-            if (win.isStuck()) {
-                // if we're sticky then reassociate window
-                // to all workspaces
-                BScreen &scr = win.screen();
-                if (scr.currentWorkspaceID() != win.workspaceNumber()) {
-                    scr.reassociateWindow(&win,
-                                          scr.currentWorkspaceID(),
-                                          true);
-                }
-            }
-        } else if ((&(win.layerSig())) == changedsub) { // layer signal
-
-            for (AtomHandlerContainerIt it= m_atomhandler.begin();
-                 it != m_atomhandler.end(); ++it) {
-                if ((*it).first->update())
-                    (*it).first->updateLayer(win);
-            }
-        } else if ((&(win.dieSig())) == changedsub) { // window death signal
-
-            for (AtomHandlerContainerIt it= m_atomhandler.begin();
-                it != m_atomhandler.end(); ++it) {
-                if ((*it).first->update())
-                    (*it).first->updateFrameClose(win);
-            }
-
-            // make sure each workspace get this
-            BScreen &scr = win.screen();
-            scr.removeWindow(&win);
-            if (FocusControl::focusedFbWindow() == &win)
-                FocusControl::setFocusedFbWindow(0);
-
-        } else if ((&(win.workspaceSig())) == changedsub) {  // workspace signal
-            for (AtomHandlerContainerIt it= m_atomhandler.begin();
-                 it != m_atomhandler.end(); ++it) {
-                if ((*it).first->update())
-                    (*it).first->updateWorkspace(win);
-            }
-        } else {
-#ifdef DEBUG
-            cerr<<__FILE__<<"("<<__LINE__<<"): WINDOW uncought signal from "<<&win<<endl;
-#endif // DEBUG
+    if (fbwin && &fbwin->stateSig() == changedsub) { // state signal
+        for (AtomHandlerContainerIt it= m_atomhandler.begin();
+             it != m_atomhandler.end(); ++it) {
+            if ((*it).first->update())
+                (*it).first->updateState(*fbwin);
+        }
+        // if window changed to iconic state
+        // add to icon list
+        if (fbwin->isIconic()) {
+            fbwin->screen().addIcon(fbwin);
+            Workspace *space = fbwin->screen().getWorkspace(fbwin->workspaceNumber());
+            if (space != 0)
+                space->removeWindow(fbwin, true);
         }
 
+        if (fbwin->isStuck()) {
+            // if we're sticky then reassociate window
+            // to all workspaces
+            BScreen &scr = fbwin->screen();
+            if (scr.currentWorkspaceID() != fbwin->workspaceNumber()) {
+                scr.reassociateWindow(fbwin,
+                                      scr.currentWorkspaceID(),
+                                      true);
+            }
+        }
+    } else if (fbwin && &fbwin->layerSig() == changedsub) { // layer signal
+        AtomHandlerContainerIt it= m_atomhandler.begin();
+        for (; it != m_atomhandler.end(); ++it) {
+            if ((*it).first->update())
+                (*it).first->updateLayer(*fbwin);
+        }
+    } else if (fbwin && &fbwin->dieSig() == changedsub) { // window death signal
+        AtomHandlerContainerIt it= m_atomhandler.begin();
+        for (; it != m_atomhandler.end(); ++it) {
+            if ((*it).first->update())
+                (*it).first->updateFrameClose(*fbwin);
+        }
+
+        // make sure each workspace get this
+        BScreen &scr = fbwin->screen();
+        scr.removeWindow(fbwin);
+        if (FocusControl::focusedFbWindow() == fbwin)
+            FocusControl::setFocusedFbWindow(0);
+    } else if (fbwin && &fbwin->workspaceSig() == changedsub) {  // workspace signal
+        for (AtomHandlerContainerIt it= m_atomhandler.begin();
+             it != m_atomhandler.end(); ++it) {
+            if ((*it).first->update())
+                (*it).first->updateWorkspace(*fbwin);
+        }
+    } else if (client && &client->dieSig() == changedsub) { // client death
+        for (AtomHandlerContainerIt it= m_atomhandler.begin();
+             it != m_atomhandler.end(); ++it) {
+            if ((*it).first->update())
+                (*it).first->updateClientClose(*client);
+        }
+
+        BScreen &screen = client->screen();
+
+        // At this point, we trust that this client is no longer in the
+        // client list of its frame (but it still has reference to the frame)
+        // We also assume that any remaining active one is the last focused one
+
+        // This is where we revert focus on window close
+        // NOWHERE ELSE!!!
+        if (FocusControl::focusedWindow() == client) {
+            FocusControl::unfocusWindow(*client);
+            // make sure nothing else uses this window before focus reverts
+            FocusControl::setFocusedWindow(0);
+        } else if (FocusControl::expectingFocus() == client) {
+            FocusControl::setExpectingFocus(0);
+            revertFocus();
+        }
+
+        screen.removeClient(*client);
     } else if (typeid(*changedsub) == typeid(BScreen::ScreenSubject)) {
         BScreen::ScreenSubject *subj = dynamic_cast<BScreen::ScreenSubject *>(changedsub);
         BScreen &screen = subj->screen();
@@ -1205,6 +1112,14 @@ void Fluxbox::update(FbTk::Subject *changedsub) {
                 if ((*it).first->update())
                     (*it).first->updateCurrentWorkspace(screen);
             }
+        } else if ((&(screen.focusedWindowSig())) == changedsub) {
+            for (AtomHandlerContainerIt it= m_atomhandler.begin();
+                 it != m_atomhandler.end(); it++) {
+                (*it).first->updateFocusedWindow(screen,
+                        (FocusControl::focusedWindow() ?
+                         FocusControl::focusedWindow()->window() :
+                         0));
+            }
         } else if ((&(screen.workspaceAreaSig())) == changedsub) {
             for (AtomHandlerContainerIt it= m_atomhandler.begin();
                  it != m_atomhandler.end(); ++it) {
@@ -1218,37 +1133,6 @@ void Fluxbox::update(FbTk::Subject *changedsub) {
                     (*it).first->updateClientList(screen);
             }
         }
-    } else if (typeid(*changedsub) == typeid(WinClient::WinClientSubj)) {
-
-        WinClient::WinClientSubj *subj = dynamic_cast<WinClient::WinClientSubj *>(changedsub);
-        WinClient &client = subj->winClient();
-
-        // TODO: don't assume it is diesig (need to fix as soon as another signal appears)
-        for (AtomHandlerContainerIt it= m_atomhandler.begin();
-             it != m_atomhandler.end(); ++it) {
-            if ((*it).first->update())
-                (*it).first->updateClientClose(client);
-        }
-
-        BScreen &screen = client.screen();
-
-        screen.removeClient(client);
-        // finaly send notify signal
-        screen.updateNetizenWindowDel(client.window());
-
-        // At this point, we trust that this client is no longer in the
-        // client list of its frame (but it still has reference to the frame)
-        // We also assume that any remaining active one is the last focused one
-
-        // This is where we revert focus on window close
-        // NOWHERE ELSE!!!
-        if (FocusControl::focusedWindow() == &client) {
-            FocusControl::unfocusWindow(client);
-            // make sure nothing else uses this window before focus reverts
-            FocusControl::setFocusedWindow(0);
-            m_revert_screen = &screen;
-            m_revert_timer.start();
-        }
     }
 }
 
@@ -1299,7 +1183,7 @@ AtomHandler* Fluxbox::getAtomHandler(const string &name) {
     return 0;
 }
 void Fluxbox::addAtomHandler(AtomHandler *atomh, const string &name) {
-    m_atomhandler[atomh]= name;;
+    m_atomhandler[atomh]= name;
 }
 
 void Fluxbox::removeAtomHandler(AtomHandler *atomh) {
@@ -1440,18 +1324,14 @@ void Fluxbox::save_rc() {
 
 /// @return filename of resource file
 string Fluxbox::getRcFilename() {
-
-    if (m_rc_file.empty()) { // set default filename
-        string defaultfile(getenv("HOME") + string("/.") + m_RC_PATH + string("/") + m_RC_INIT_FILE);
-        return defaultfile;
-    }
-
+    if (m_rc_file.empty())
+        return getDefaultDataFilename(m_RC_INIT_FILE);
     return m_rc_file;
 }
 
 /// Provides default filename of data file
-void Fluxbox::getDefaultDataFilename(const char *name, string &filename) {
-    filename = string(getenv("HOME") + string("/.") + m_RC_PATH + string("/") + name);
+string Fluxbox::getDefaultDataFilename(const char *name) const {
+    return (getenv("HOME") + string("/.") + m_RC_PATH + string("/") + name);
 }
 
 /// loads resources
@@ -1480,8 +1360,7 @@ void Fluxbox::load_rc() {
     if (!m_rc_slitlistfile->empty()) {
         *m_rc_slitlistfile = StringUtil::expandFilename(*m_rc_slitlistfile);
     } else {
-        string filename;
-        getDefaultDataFilename("slitlist", filename);
+        string filename = getDefaultDataFilename("slitlist");
         m_rc_slitlistfile.setFromString(filename.c_str());
     }
 
@@ -1492,12 +1371,6 @@ void Fluxbox::load_rc() {
 
     if (m_rc_stylefile->empty())
         *m_rc_stylefile = DEFAULTSTYLE;
-
-    if (!Workspace::loadGroups(*m_rc_groupfile)) {
-#ifdef DEBUG
-        cerr<<_FB_CONSOLETEXT(Fluxbox, CantLoadGroupFile, "Failed to load groupfile", "Couldn't load the groupfile")<<": "<<*m_rc_groupfile<<endl;
-#endif // DEBUG
-    }
 }
 
 void Fluxbox::load_rc(BScreen &screen) {
@@ -1537,17 +1410,6 @@ void Fluxbox::load_rc(BScreen &screen) {
 
     }
 
-    FbTk::Image::removeAllSearchPaths();
-    sprintf(name_lookup, "session.screen%d.imageSearchPath", screen_number);
-    sprintf(class_lookup, "Session.Screen%d.imageSearchPath", screen_number);
-    if (XrmGetResource(*database, name_lookup, class_lookup, &value_type,
-                       &value) && value.addr) {
-        vector<string> paths;
-        StringUtil::stringtok(paths, value.addr, ", ");
-        for (size_t i = 0; i < paths.size(); ++i)
-            FbTk::Image::addSearchPath(paths[i]);
-    }
-
     if (!dbfile.empty()) {
         if (!m_screen_rm.load(dbfile.c_str())) {
             cerr<<_FB_CONSOLETEXT(Fluxbox, CantLoadRCFile, "Failed to load database", "Failed trying to read rc file")<<":"<<dbfile<<endl;
@@ -1581,7 +1443,7 @@ void Fluxbox::real_reconfigure() {
     for_each(m_screen_list.begin(), m_screen_list.end(), mem_fun(&BScreen::reconfigure));
 
     //reconfigure keys
-    m_key->reconfigure(StringUtil::expandFilename(*m_rc_keyfile).c_str());
+    m_key->reconfigure();
 
     // and atomhandlers
     for (AtomHandlerContainerIt it= m_atomhandler.begin();
@@ -1605,123 +1467,39 @@ BScreen *Fluxbox::findScreen(int id) {
     return *it;
 }
 
-bool Fluxbox::menuTimestampsChanged() const {
-    list<MenuTimestamp *>::const_iterator it = m_menu_timestamps.begin();
-    list<MenuTimestamp *>::const_iterator it_end = m_menu_timestamps.end();
-    for (; it != it_end; ++it) {
-
-        time_t timestamp = FbTk::FileUtil::getLastStatusChangeTimestamp((*it)->filename.c_str());
-
-        if (timestamp >= 0) {
-            if (timestamp != (*it)->timestamp)
-                return true;
-        } else
-            return true;
-    }
-
-    // no timestamp changed
-    return false;
-}
-
-void Fluxbox::hideExtraMenus(BScreen &screen) {
-
-#ifdef USE_TOOLBAR
-        // hide toolbar that matches screen
-        for (size_t toolbar = 0; toolbar < m_toolbars.size(); ++toolbar) {
-            if (&(m_toolbars[toolbar]->screen()) == &screen)
-                m_toolbars[toolbar]->menu().hide();
-        }
-
-#endif // USE_TOOLBAR
-
-}
-
-void Fluxbox::rereadMenu(bool show_after_reread) {
-    m_reread_menu_wait = true;
-    m_show_menu_after_reread = show_after_reread;
-    m_reconfig_timer.start();
-}
-
-
-void Fluxbox::real_rereadMenu() {
-
-    clearMenuFilenames();
-
-    for_each(m_screen_list.begin(),
-             m_screen_list.end(),
-             mem_fun(&BScreen::rereadMenu));
-
-    if(m_show_menu_after_reread) {
-
-        FbCommands::ShowRootMenuCmd showcmd;
-        showcmd.execute();
-
-        m_show_menu_after_reread = false;
-    }
-}
-
-void Fluxbox::saveMenuFilename(const char *filename) {
-    if (filename == 0)
-        return;
-
-    bool found = false;
-
-    list<MenuTimestamp *>::iterator it = m_menu_timestamps.begin();
-    list<MenuTimestamp *>::iterator it_end = m_menu_timestamps.end();
-    for (; it != it_end; ++it) {
-        if ((*it)->filename == filename) {
-            found = true;
-            break;
-        }
-    }
-
-    if (! found) {
-        time_t timestamp = FbTk::FileUtil::getLastStatusChangeTimestamp(filename);
-
-        if (timestamp >= 0) {
-            MenuTimestamp *ts = new MenuTimestamp;
-
-            ts->filename = filename;
-            ts->timestamp = timestamp;
-
-            m_menu_timestamps.push_back(ts);
-        }
-    }
-}
-
-void Fluxbox::clearMenuFilenames() {
-    while(!m_menu_timestamps.empty()) {
-        delete m_menu_timestamps.back();
-        m_menu_timestamps.pop_back();
-    }
-}
-
 void Fluxbox::timed_reconfigure() {
     if (m_reconfigure_wait)
         real_reconfigure();
 
-    if (m_reread_menu_wait)
-        real_rereadMenu();
-
-    m_reconfigure_wait = m_reread_menu_wait = false;
+    m_reconfigure_wait = false;
 }
 
-void Fluxbox::revert_focus() {
-    if (!m_revert_screen || FocusControl::focusedWindow() ||
-            FbTk::Menu::focused() || m_showing_dialog)
-        return;
+void Fluxbox::revertFocus() {
+    bool revert = m_keyscreen && !m_showing_dialog;
 
-    Window win;
-    int revert;
-    Display *disp = display();
+    if (revert) {
+        // see if there are any more focus events in the queue
+        XEvent ev;
+        while (XCheckMaskEvent(display(), FocusChangeMask, &ev))
+            handleEvent(&ev);
+        if (FocusControl::focusedWindow() || FocusControl::expectingFocus())
+            return; // already handled
 
-    XGetInputFocus(disp, &win, &revert);
+        Window win;
+        int blah;
+        XGetInputFocus(display(), &win, &blah);
 
-    // we only want to revert focus if it's left dangling, as some other
-    // application may have set the focus to an unmanaged window
-    if (win == None || win == PointerRoot ||
-            win == m_revert_screen->rootWindow().window())
-        FocusControl::revertFocus(*m_revert_screen);
+        // we only want to revert focus if it's left dangling, as some other
+        // application may have set the focus to an unmanaged window
+        if (win != None && win != PointerRoot && !searchWindow(win) &&
+            win != m_keyscreen->rootWindow().window())
+            revert = false;
+    }
+
+    if (revert)
+        FocusControl::revertFocus(*m_keyscreen);
+    else
+        FocusControl::setFocusedWindow(0);
 }
 
 bool Fluxbox::validateClient(const WinClient *client) const {
@@ -1733,25 +1511,6 @@ bool Fluxbox::validateClient(const WinClient *client) const {
     return it != m_window_search.end();
 }
 
-void Fluxbox::updateFocusedWindow(BScreen *screen, BScreen *old_screen) {
-    if (screen != 0) {
-        screen->updateNetizenWindowFocus();
-        for (AtomHandlerContainerIt it= m_atomhandler.begin();
-             it != m_atomhandler.end(); it++) {
-            (*it).first->updateFocusedWindow(*screen, (FocusControl::focusedWindow() ?
-                                                       FocusControl::focusedWindow()->window() :
-                                                       0));
-        }
-    }
-
-    if (old_screen && old_screen != screen) {
-        old_screen->updateNetizenWindowFocus();
-        for (AtomHandlerContainerIt it= m_atomhandler.begin();
-             it != m_atomhandler.end(); it++)
-            (*it).first->updateFocusedWindow(*old_screen, 0);
-    }
-}
-
 void Fluxbox::updateFrameExtents(FluxboxWindow &win) {
     AtomHandlerContainerIt it = m_atomhandler.begin();
     AtomHandlerContainerIt it_end = m_atomhandler.end();
@@ -1759,22 +1518,3 @@ void Fluxbox::updateFrameExtents(FluxboxWindow &win) {
         (*it).first->updateFrameExtents(win);
     }
 }
-
-unsigned int Fluxbox::getModKey() const {
-    if (!(m_rc_mod_key->c_str()))
-        return 0;
-    else
-        return FbTk::KeyUtil::instance().getModifier(m_rc_mod_key->c_str());
-}
-
-void Fluxbox::setModKey(const char* modkeyname) {
-
-    if (!modkeyname)
-        return;
-
-    unsigned int modkey = FbTk::KeyUtil::instance().getModifier(modkeyname);
-
-    if (modkey > 0) {
-        m_rc_mod_key = modkeyname;
-    }
-}
